import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.HashMap;

import javax.swing.JFrame;
import javax.swing.JPanel;

import com.sun.star.container.NoSuchElementException;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.sheet.XSheetCellCursor;
import com.sun.star.sheet.XSpreadsheet;
import com.sun.star.sheet.XSpreadsheets;
import com.sun.star.table.XCell;
import com.sun.star.text.XText;
import com.sun.star.uno.UnoRuntime;

import ag.ion.bion.officelayer.NativeView;
import ag.ion.bion.officelayer.application.ILazyApplicationInfo;
import ag.ion.bion.officelayer.application.IOfficeApplication;
import ag.ion.bion.officelayer.application.OfficeApplicationException;
import ag.ion.bion.officelayer.application.OfficeApplicationRuntime;
import ag.ion.bion.officelayer.desktop.DesktopException;
import ag.ion.bion.officelayer.desktop.IFrame;
import ag.ion.bion.officelayer.document.DocumentDescriptor;
import ag.ion.bion.officelayer.document.DocumentException;
import ag.ion.bion.officelayer.document.IDocument;
import ag.ion.bion.officelayer.internal.application.ApplicationAssistant;
import ag.ion.bion.officelayer.internal.spreadsheet.SpreadsheetDocument;
import ag.ion.bion.officelayer.spreadsheet.ISpreadsheetDocument;
import ag.ion.bion.officelayer.util.Assert;
import ag.ion.noa.NOAException;

/**
 * Jochen Strk
 * jstaerk@usegroup.de
 * 
 * Nice Office Access (NOA)-Libre, 
 * the Java library for LibreOffice/UNO access: 
 * http://code.google.com/p/noa-libre  
 * 
 * My open source accounting software based on NOA-Libre: 
 * http://www.gnuaccounting.org
 */


 
/**
 * LibreOffice Embed Window Example - LOEWE
 * */
public class Loewe extends JFrame {
	
	private JPanel NOAPanel;
	private ApplicationAssistant ass;
	private IOfficeApplication officeApp=null;
	private IFrame officeFrame;
	private ISpreadsheetDocument document;

	public Loewe() {
		setSize(500,500);
		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		NOAPanel=new JPanel();
		getContentPane().add(NOAPanel);
		getContentPane().setLayout(new GridLayout());
		addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {

				if (officeApp!=null) {
					try {
						officeApp.deactivate();
					} catch (OfficeApplicationException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
				super.windowClosing(e);
			}
			
		});
		
		try {
			ass=new ApplicationAssistant(System.getProperty("user.dir")+File.separator+"lib");
			ILazyApplicationInfo[] ila=ass.getLocalApplications();
			HashMap<String, String> configuration=new HashMap();
			configuration.put(IOfficeApplication.APPLICATION_HOME_KEY, ila[0].getHome());
			configuration.put(IOfficeApplication.APPLICATION_TYPE_KEY, IOfficeApplication.LOCAL_APPLICATION);
			officeApp=OfficeApplicationRuntime.getApplication(configuration);
			officeApp.activate();
			
			final NativeView nat = new NativeView(System.getProperty("user.dir")+File.separator+"lib");
			nat.setSize(NOAPanel.getWidth(),NOAPanel.getHeight());
			NOAPanel.addComponentListener(new ComponentAdapter() {

				@Override
				public void componentResized(ComponentEvent e) {
					
					super.componentResized(e);
					nat.setPreferredSize(new Dimension(NOAPanel.getWidth(),NOAPanel.getHeight()));
					NOAPanel.getLayout().layoutContainer(NOAPanel);
				}
				
			});
			NOAPanel.add(nat);
			NOAPanel.setVisible(true);
			officeFrame=officeApp.getDesktopService().constructNewOfficeFrame(nat);
			document=(ISpreadsheetDocument)officeApp.getDocumentService().constructNewDocument(officeFrame, IDocument.CALC, DocumentDescriptor.DEFAULT);
			XSpreadsheets spreadsheets = document.getSpreadsheetDocument().getSheets();
			try {
				XSpreadsheet spreadsheet1 = (XSpreadsheet) UnoRuntime.queryInterface(XSpreadsheet.class, spreadsheets.getByName(spreadsheets.getElementNames()[0]));
				XSheetCellCursor cellCursor = spreadsheet1.createCursor();
				XCell cell=null;
				cell=cellCursor.getCellByPosition(0, 1);
				cell.setValue(12);
				cell=cellCursor.getCellByPosition(0, 2);
				cell.setValue(10.2);
				cell=cellCursor.getCellByPosition(1, 1);
				cell.setValue(22.2);
				cell=cellCursor.getCellByPosition(0, 0);
				XText cellText=(XText) UnoRuntime.queryInterface(XText.class, cell);
				cellText.setString("Accounts (receivables)");
				
				
			} catch (NoSuchElementException | WrappedTargetException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
//			document=officeApp.getDocumentService().loadDocument(officeFrame, "C:\\Users\\jstaerk\\temp\\test.odt");
 catch (IndexOutOfBoundsException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
		} catch (OfficeApplicationException | DesktopException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (NOAException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

		Loewe l=new Loewe();
	}

}
